package com.ruoyi.interfaces.mapper;

import com.ruoyi.interfaces.domain.PtDataImp;
import com.ruoyi.interfaces.domain.PtDataImpField;

import java.util.List;

public interface PtDataImpMapper {
    int insert(PtDataImp p);

    int update(PtDataImp p);

    int delete(String id);

    List<PtDataImp> selectAll(PtDataImp p);

    PtDataImp findByOne(String id);

    int insertPtDatImpField(PtDataImpField p);

    int updatePtDatImpField(PtDataImpField p);

    List<PtDataImpField> selectAllPtDatImpField(PtDataImpField p);

    int deletePtDataImpField(PtDataImpField p);

//    int insertTest(Test test);
}
